import 'dart:async';
import 'dart:io';
import 'dart:convert';
import 'package:dio/dio.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/serializer.dart';

import 'package:flagr/model/create_segment_request.dart';
import 'package:flagr/model/put_segment_request.dart';
import 'package:flagr/model/put_segment_reorder_request.dart';
import 'package:flagr/model/segment.dart';
import 'package:flagr/model/error.dart';

class SegmentApi {
  final Dio _dio;
  Serializers _serializers;

  SegmentApi(this._dio, this._serializers);

  ///
  ///
  ///
  Future<Response<Segment>> createSegment(
    int flagID,
    CreateSegmentRequest body, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/segments"
        .replaceAll("{" r'flagID' "}", flagID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = ["application/json"];

    var serializedBody = _serializers.serialize(body);
    var jsonbody = json.encode(serializedBody);
    bodyData = jsonbody;

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'post'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(Segment);
      var data = _serializers.deserializeWith<Segment>(serializer,
          response.data is String ? jsonDecode(response.data) : response.data);

      return Response<Segment>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  ///
  ///
  ///
  Future<Response> deleteSegment(
    int flagID,
    int segmentID, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/segments/{segmentID}"
        .replaceAll("{" r'flagID' "}", flagID.toString())
        .replaceAll("{" r'segmentID' "}", segmentID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = [];

    return _dio.request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'delete'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    );
  }

  ///
  ///
  ///
  Future<Response<List<Segment>>> findSegments(
    int flagID, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/segments"
        .replaceAll("{" r'flagID' "}", flagID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = [];

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'get'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      final FullType type =
          const FullType(BuiltList, const [const FullType(Segment)]);
      BuiltList<Segment> dataList = _serializers.deserialize(
          response.data is String ? jsonDecode(response.data) : response.data,
          specifiedType: type);
      var data = dataList.toList();

      return Response<List<Segment>>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  ///
  ///
  ///
  Future<Response<Segment>> putSegment(
    int flagID,
    int segmentID,
    PutSegmentRequest body, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/segments/{segmentID}"
        .replaceAll("{" r'flagID' "}", flagID.toString())
        .replaceAll("{" r'segmentID' "}", segmentID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = ["application/json"];

    var serializedBody = _serializers.serialize(body);
    var jsonbody = json.encode(serializedBody);
    bodyData = jsonbody;

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'put'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(Segment);
      var data = _serializers.deserializeWith<Segment>(serializer,
          response.data is String ? jsonDecode(response.data) : response.data);

      return Response<Segment>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  ///
  ///
  ///
  Future<Response> putSegmentsReorder(
    int flagID,
    PutSegmentReorderRequest body, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/segments/reorder"
        .replaceAll("{" r'flagID' "}", flagID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = ["application/json"];

    var serializedBody = _serializers.serialize(body);
    var jsonbody = json.encode(serializedBody);
    bodyData = jsonbody;

    return _dio.request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'put'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    );
  }
}
