import 'dart:async';
import 'dart:io';
import 'dart:convert';
import 'package:dio/dio.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/serializer.dart';

import 'package:flagr/model/create_variant_request.dart';
import 'package:flagr/model/variant.dart';
import 'package:flagr/model/error.dart';
import 'package:flagr/model/put_variant_request.dart';

class VariantApi {
  final Dio _dio;
  Serializers _serializers;

  VariantApi(this._dio, this._serializers);

  ///
  ///
  ///
  Future<Response<Variant>> createVariant(
    int flagID,
    CreateVariantRequest body, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/variants"
        .replaceAll("{" r'flagID' "}", flagID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = ["application/json"];

    var serializedBody = _serializers.serialize(body);
    var jsonbody = json.encode(serializedBody);
    bodyData = jsonbody;

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'post'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(Variant);
      var data = _serializers.deserializeWith<Variant>(serializer,
          response.data is String ? jsonDecode(response.data) : response.data);

      return Response<Variant>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  ///
  ///
  ///
  Future<Response> deleteVariant(
    int flagID,
    int variantID, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/variants/{variantID}"
        .replaceAll("{" r'flagID' "}", flagID.toString())
        .replaceAll("{" r'variantID' "}", variantID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = [];

    return _dio.request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'delete'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    );
  }

  ///
  ///
  ///
  Future<Response<List<Variant>>> findVariants(
    int flagID, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/variants"
        .replaceAll("{" r'flagID' "}", flagID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = [];

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'get'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      final FullType type =
          const FullType(BuiltList, const [const FullType(Variant)]);
      BuiltList<Variant> dataList = _serializers.deserialize(
          response.data is String ? jsonDecode(response.data) : response.data,
          specifiedType: type);
      var data = dataList.toList();

      return Response<List<Variant>>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  ///
  ///
  ///
  Future<Response<Variant>> putVariant(
    int flagID,
    int variantID,
    PutVariantRequest body, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/flags/{flagID}/variants/{variantID}"
        .replaceAll("{" r'flagID' "}", flagID.toString())
        .replaceAll("{" r'variantID' "}", variantID.toString());

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = ["application/json"];

    var serializedBody = _serializers.serialize(body);
    var jsonbody = json.encode(serializedBody);
    bodyData = jsonbody;

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'put'.toUpperCase(),
        headers: headerParams,
        extra: {
          'secure': [],
        },
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(Variant);
      var data = _serializers.deserializeWith<Variant>(serializer,
          response.data is String ? jsonDecode(response.data) : response.data);

      return Response<Variant>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }
}
